@@ -59,6 +59,8 @@ static const int FlickThreshold = 15;
 // will ensure the Flickable retains the grab on consecutive flicks.
 static const int RetainGrabVelocity = 100;
 
+static const int MovementEndingTimerInterval = 100;
+
 static qreal EaseOvershoot(qreal t) {
     return qAtan(t);
 }
@@ -218,7 +220,7 @@ QQuickFlickablePrivate::QQuickFlickablePrivate()
     , hMoved(false), vMoved(false)
     , stealMouse(false), pressed(false)
     , scrollingPhase(false), interactive(true), calcVelocity(false)
-    , pixelAligned(false), replayingPressEvent(false)
+    , pixelAligned(false)
     , lastPosTime(-1)
     , lastPressTime(0)
     , deceleration(QML_FLICK_DEFAULTDECELERATION)
@@ -238,6 +240,8 @@ void QQuickFlickablePrivate::init()
     contentItem->setParentItem(q);
     qmlobject_connect(&timeline, QQuickTimeLine, SIGNAL(completed()),
                       q, QQuickFlickable, SLOT(timelineCompleted()))
+    qmlobject_connect(&velocityTimeline, QQuickTimeLine, SIGNAL(completed()),
+                      q, QQuickFlickable, SLOT(velocityTimelineCompleted()))
     q->setAcceptedMouseButtons(Qt::LeftButton);
     q->setFiltersChildMouseEvents(true);
     QQuickItemPrivate *viewportPrivate = QQuickItemPrivate::get(contentItem);
@@ -344,10 +348,12 @@ bool QQuickFlickablePrivate::flick(AxisData &data, qreal minExtent, qreal maxExt
         accel = v2 / (2.0f * qAbs(dist));
 
         resetTimeline(data);
-        if (boundsBehavior & QQuickFlickable::OvershootBounds)
-            timeline.accel(data.move, v, accel);
-        else
-            timeline.accel(data.move, v, accel, maxDistance);
+        if (!data.inOvershoot) {
+            if (boundsBehavior & QQuickFlickable::OvershootBounds)
+                timeline.accel(data.move, v, accel);
+            else
+                timeline.accel(data.move, v, accel, maxDistance);
+        }
         timeline.callback(QQuickTimeLineCallback(&data.move, fixupCallback, this));
 
         if (&data == &hData)
@@ -433,12 +439,12 @@ void QQuickFlickablePrivate::clearTimeline()
 
 void QQuickFlickablePrivate::fixup(AxisData &data, qreal minExtent, qreal maxExtent)
 {
-    if (data.move.value() > minExtent || maxExtent > minExtent) {
+    if (data.move.value() >= minExtent || maxExtent > minExtent) {
         resetTimeline(data);
         if (data.move.value() != minExtent) {
             adjustContentPos(data, minExtent);
         }
-    } else if (data.move.value() < maxExtent) {
+    } else if (data.move.value() <= maxExtent) {
         resetTimeline(data);
         adjustContentPos(data, maxExtent);
     } else if (-qRound(-data.move.value()) != data.move.value()) {
@@ -619,7 +625,7 @@ is finished.
     \qmlsignal QtQuick::Flickable::movementStarted()
 
     This signal is emitted when the view begins moving due to user
-    interaction.
+    interaction or a generated flick().
 
     The corresponding handler is \c onMovementStarted.
 */
@@ -628,9 +634,9 @@ is finished.
     \qmlsignal QtQuick::Flickable::movementEnded()
 
     This signal is emitted when the view stops moving due to user
-    interaction.  If a flick was generated, this signal will
+    interaction or a generated flick().  If a flick was active, this signal will
     be emitted once the flick stops.  If a flick was not
-    generated, this signal will be emitted when the
+    active, this signal will be emitted when the
     user stops dragging - i.e. a mouse or touch release.
 
     The corresponding handler is \c onMovementEnded.
@@ -1194,6 +1200,8 @@ void QQuickFlickablePrivate::drag(qint64 currentTimestamp, QEvent::Type eventTyp
         hData.velocity = 0;
     }
 
+    if (momentum && !hData.flicking && !vData.flicking)
+        flickingStarted(hData.velocity != 0, vData.velocity != 0);
     draggingStarting();
 
     if ((hMoved && !prevHMoved) || (vMoved && !prevVMoved))
@@ -1347,7 +1355,7 @@ void QQuickFlickable::mouseReleaseEvent(QMouseEvent *event)
             if (window() && window()->mouseGrabberItem()) {
                 QPointF localPos = window()->mouseGrabberItem()->mapFromScene(event->windowPos());
                 QScopedPointer<QMouseEvent> mouseEvent(QQuickWindowPrivate::cloneMouseEvent(event, &localPos));
-                window()->sendEvent(window()->mouseGrabberItem(), mouseEvent.data());
+                QCoreApplication::sendEvent(window(), mouseEvent.data());
             }
 
             // And the event has been consumed
@@ -1382,9 +1390,16 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
         d->timer.start();
         d->maybeBeginDrag(currentTimestamp, event->posF());
         break;
+    case Qt::NoScrollPhase: // default phase with an ordinary wheel mouse
     case Qt::ScrollUpdate:
+        if (d->scrollingPhase)
+            d->pressed = true;
+#ifdef Q_OS_OSX
+        d->movementEndingTimer.start(MovementEndingTimerInterval, this);
+#endif
         break;
     case Qt::ScrollEnd:
+        d->pressed = false;
         d->scrollingPhase = false;
         d->draggingEnding();
         event->accept();
@@ -1407,7 +1422,6 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
                 valid = true;
             }
             if (valid) {
-                d->vData.flicking = false;
                 d->flickY(d->vData.velocity);
                 d->flickingStarted(false, true);
                 if (d->vData.flicking) {
@@ -1427,7 +1441,6 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
                 valid = true;
             }
             if (valid) {
-                d->hData.flicking = false;
                 d->flickX(d->hData.velocity);
                 d->flickingStarted(true, false);
                 if (d->hData.flicking) {
@@ -1451,6 +1464,7 @@ void QQuickFlickable::wheelEvent(QWheelEvent *event)
         d->lastPosTime = currentTimestamp;
         d->accumulatedWheelPixelDelta += QVector2D(event->pixelDelta());
         d->drag(currentTimestamp, event->type(), event->posF(), d->accumulatedWheelPixelDelta, true, !d->scrollingPhase, true, velocity);
+        event->accept();
     }
 
     if (!event->isAccepted())
@@ -1539,6 +1553,12 @@ void QQuickFlickable::timerEvent(QTimerEvent *event)
         if (d->delayedPressEvent) {
             d->replayDelayedPress();
         }
+    } else if (event->timerId() == d->movementEndingTimer.timerId()) {
+        d->movementEndingTimer.stop();
+        d->pressed = false;
+        d->stealMouse = false;
+        if (!d->velocityTimeline.isActive())
+            movementEnding(true, true);
     }
 }
 
@@ -1639,10 +1659,8 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
     bool changed = false;
     if (newGeometry.width() != oldGeometry.width()) {
         changed = true; // we must update visualArea.widthRatio
-        if (d->hData.viewSize < 0) {
+        if (d->hData.viewSize < 0)
             d->contentItem->setWidth(width());
-            emit contentWidthChanged();
-        }
         // Make sure that we're entirely in view.
         if (!d->pressed && !d->hData.moving && !d->vData.moving) {
             d->fixupMode = QQuickFlickablePrivate::Immediate;
@@ -1651,10 +1669,8 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
     }
     if (newGeometry.height() != oldGeometry.height()) {
         changed = true; // we must update visualArea.heightRatio
-        if (d->vData.viewSize < 0) {
+        if (d->vData.viewSize < 0)
             d->contentItem->setHeight(height());
-            emit contentHeightChanged();
-        }
         // Make sure that we're entirely in view.
         if (!d->pressed && !d->hData.moving && !d->vData.moving) {
             d->fixupMode = QQuickFlickablePrivate::Immediate;
@@ -1670,6 +1686,9 @@ void QQuickFlickable::geometryChanged(const QRectF &newGeometry,
     \qmlmethod QtQuick::Flickable::flick(qreal xVelocity, qreal yVelocity)
 
     Flicks the content with \a xVelocity horizontally and \a yVelocity vertically in pixels/sec.
+
+    Calling this method will update the corresponding moving and flicking properties and signals,
+    just like a real flick.
 */
 
 void QQuickFlickable::flick(qreal xVelocity, qreal yVelocity)
@@ -1679,8 +1698,15 @@ void QQuickFlickable::flick(qreal xVelocity, qreal yVelocity)
     d->vData.reset();
     d->hData.velocity = xVelocity;
     d->vData.velocity = yVelocity;
+
     bool flickedX = d->flickX(xVelocity);
     bool flickedY = d->flickY(yVelocity);
+
+    if (flickedX)
+        d->hMoved = true;
+    if (flickedY)
+        d->vMoved = true;
+    movementStarting();
     d->flickingStarted(flickedX, flickedY);
 }
 
@@ -2195,10 +2221,6 @@ bool QQuickFlickable::sendMouseEvent(QQuickItem *item, QMouseEvent *event)
             d->handleMouseMoveEvent(mouseEvent.data());
             break;
         case QEvent::MouseButtonPress:
-            // Don't process a replayed event during replay
-            if (d->replayingPressEvent)
-                return false;
-
             d->handleMousePressEvent(mouseEvent.data());
             d->captureDelayedPress(item, event);
             stealThisEvent = d->stealMouse;   // Update stealThisEvent in case changed by function call above
@@ -2464,6 +2486,22 @@ bool QQuickFlickable::isMovingVertically() const
     return d->vData.moving;
 }
 
+void QQuickFlickable::velocityTimelineCompleted()
+{
+    Q_D(QQuickFlickable);
+    if ( (d->hData.transitionToBounds && d->hData.transitionToBounds->isActive())
+         || (d->vData.transitionToBounds && d->vData.transitionToBounds->isActive()) ) {
+        return;
+    }
+    // With subclasses such as GridView, velocityTimeline.completed is emitted repeatedly:
+    // for example setting currentIndex results in a visual "flick" which the user
+    // didn't initiate directly. We don't want to end movement repeatedly, and in
+    // that case movementEnding will happen after the sequence of movements ends.
+    if (d->vData.flicking)
+        movementEnding();
+    d->updateBeginningEnd();
+}
+
 void QQuickFlickable::timelineCompleted()
 {
     Q_D(QQuickFlickable);
