@@ -126,9 +126,8 @@ void QQuickAnimatorProxyJob::updateState(QAbstractAnimationJob::State newState,
             m_controller->startJob(this, m_job);
     } else if (newState == Stopped) {
         syncBackCurrentValues();
-        if (m_internalState == State_Starting)
-            m_internalState = State_Stopped;
-        else if (m_controller) {
+        m_internalState = State_Stopped;
+        if (m_controller) {
             m_controller->stopJob(this, m_job);
         }
     }
@@ -161,6 +160,8 @@ void QQuickAnimatorProxyJob::setWindow(QQuickWindow *window)
         // Upon leaving a window, we reset the controller. This means that
         // animators will only enter the Starting phase and won't be making
         // calls to QQuickAnimatorController::startjob().
+        if (m_controller)
+            m_controller->proxyWasDestroyed(this);
         m_controller = 0;
 
     } else if (!m_controller && m_job) {
@@ -230,6 +231,10 @@ void QQuickAnimatorJob::debugAnimation(QDebug d) const
       << "target:" << m_target << "value:" << m_value;
 }
 
+qreal QQuickAnimatorJob::progress(int time) const
+{
+    return m_easing.valueForProgress((m_duration == 0) ? qreal(1) : qreal(time) / qreal(m_duration));
+}
 qreal QQuickAnimatorJob::value() const
 {
     qreal v;
@@ -380,7 +385,7 @@ void QQuickXAnimatorJob::updateCurrentTime(int time)
         return;
     Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_helper->dx = m_value;
     m_helper->wasChanged = true;
 }
@@ -397,7 +402,7 @@ void QQuickYAnimatorJob::updateCurrentTime(int time)
         return;
     Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_helper->dy = m_value;
     m_helper->wasChanged = true;
 }
@@ -467,7 +472,7 @@ void QQuickOpacityAnimatorJob::updateCurrentTime(int time)
         return;
     Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_opacityNode->setOpacity(m_value);
 }
 
@@ -483,7 +488,7 @@ void QQuickScaleAnimatorJob::updateCurrentTime(int time)
         return;
     Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_helper->scale = m_value;
     m_helper->wasChanged = true;
 }
@@ -503,7 +508,8 @@ void QQuickRotationAnimatorJob::updateCurrentTime(int time)
         return;
     Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    float t =  m_easing.valueForProgress(time / (qreal) m_duration);
+    float t = progress(time);
+
     switch (m_direction) {
     case QQuickRotationAnimator::Clockwise:
         m_value = _q_interpolateClockwiseRotation(m_from, m_to, t).toFloat();
@@ -586,7 +592,7 @@ void QQuickUniformAnimatorJob::updateCurrentTime(int time)
     if (!m_node || m_uniformIndex == -1 || m_uniformType == -1)
         return;
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
 
     QQuickShaderEffectMaterial *material =
             static_cast<QQuickShaderEffectMaterial *>(m_node->material());
