@@ -1,7 +1,8 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Copyright (C) 2015 BasysKom GmbH.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,9 +11,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +24,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -39,6 +40,7 @@
 #include <QtGui/QGuiApplication>
 #include <QtGui/qdesktopservices.h>
 #include <QtGui/qfontdatabase.h>
+#include <QtGui/qstylehints.h>
 
 #include <private/qv4engine_p.h>
 #include <private/qv4object_p.h>
@@ -97,6 +99,11 @@ public:
         return QVariant(QColor::fromHslF(h, s, l, a));
     }
 
+    QVariant fromHsvF(double h, double s, double v, double a)
+    {
+        return QVariant(QColor::fromHsvF(h, s, v, a));
+    }
+
     QVariant lighter(const QVariant &var, qreal factor)
     {
         QColor color = var.value<QColor>();
@@ -261,13 +268,11 @@ public:
         return QMatrix4x4();
     }
 
-    static QFont fontFromObject(QQmlV4Handle object, QV8Engine *e, bool *ok)
+    static QFont fontFromObject(QQmlV4Handle object, QV4::ExecutionEngine *v4, bool *ok)
     {
-        Q_UNUSED(e);
-
-        if (ok) *ok = false;
+        if (ok)
+            *ok = false;
         QFont retn;
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(e);
         QV4::Scope scope(v4);
         QV4::ScopedObject obj(scope, object);
         if (!obj) {
@@ -278,17 +283,18 @@ public:
 
         QV4::ScopedString s(scope);
 
-        QV4::ScopedValue vbold(scope, obj->get((s = v4->newString(QStringLiteral("bold"))).getPointer()));
-        QV4::ScopedValue vcap(scope, obj->get((s = v4->newString(QStringLiteral("capitalization"))).getPointer()));
-        QV4::ScopedValue vfam(scope, obj->get((s = v4->newString(QStringLiteral("family"))).getPointer()));
-        QV4::ScopedValue vital(scope, obj->get((s = v4->newString(QStringLiteral("italic"))).getPointer()));
-        QV4::ScopedValue vlspac(scope, obj->get((s = v4->newString(QStringLiteral("letterSpacing"))).getPointer()));
-        QV4::ScopedValue vpixsz(scope, obj->get((s = v4->newString(QStringLiteral("pixelSize"))).getPointer()));
-        QV4::ScopedValue vpntsz(scope, obj->get((s = v4->newString(QStringLiteral("pointSize"))).getPointer()));
-        QV4::ScopedValue vstrk(scope, obj->get((s = v4->newString(QStringLiteral("strikeout"))).getPointer()));
-        QV4::ScopedValue vundl(scope, obj->get((s = v4->newString(QStringLiteral("underline"))).getPointer()));
-        QV4::ScopedValue vweight(scope, obj->get((s = v4->newString(QStringLiteral("weight"))).getPointer()));
-        QV4::ScopedValue vwspac(scope, obj->get((s = v4->newString(QStringLiteral("wordSpacing"))).getPointer()));
+        QV4::ScopedValue vbold(scope, obj->get((s = v4->newString(QStringLiteral("bold")))));
+        QV4::ScopedValue vcap(scope, obj->get((s = v4->newString(QStringLiteral("capitalization")))));
+        QV4::ScopedValue vfam(scope, obj->get((s = v4->newString(QStringLiteral("family")))));
+        QV4::ScopedValue vstyle(scope, obj->get((s = v4->newString(QStringLiteral("styleName")))));
+        QV4::ScopedValue vital(scope, obj->get((s = v4->newString(QStringLiteral("italic")))));
+        QV4::ScopedValue vlspac(scope, obj->get((s = v4->newString(QStringLiteral("letterSpacing")))));
+        QV4::ScopedValue vpixsz(scope, obj->get((s = v4->newString(QStringLiteral("pixelSize")))));
+        QV4::ScopedValue vpntsz(scope, obj->get((s = v4->newString(QStringLiteral("pointSize")))));
+        QV4::ScopedValue vstrk(scope, obj->get((s = v4->newString(QStringLiteral("strikeout")))));
+        QV4::ScopedValue vundl(scope, obj->get((s = v4->newString(QStringLiteral("underline")))));
+        QV4::ScopedValue vweight(scope, obj->get((s = v4->newString(QStringLiteral("weight")))));
+        QV4::ScopedValue vwspac(scope, obj->get((s = v4->newString(QStringLiteral("wordSpacing")))));
 
         // pull out the values, set ok to true if at least one valid field is given.
         if (vbold->isBoolean()) {
@@ -303,6 +309,10 @@ public:
             retn.setFamily(vfam->toQString());
             if (ok) *ok = true;
         }
+        if (vstyle->isString()) {
+            retn.setStyleName(vstyle->toQString());
+            if (ok) *ok = true;
+        }
         if (vital->isBoolean()) {
             retn.setItalic(vital->booleanValue());
             if (ok) *ok = true;
@@ -339,10 +349,10 @@ public:
         return retn;
     }
 
-    static QMatrix4x4 matrix4x4FromObject(QQmlV4Handle object, QV8Engine *e, bool *ok)
+    static QMatrix4x4 matrix4x4FromObject(QQmlV4Handle object, QV4::ExecutionEngine *v4, bool *ok)
     {
-        if (ok) *ok = false;
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(e);
+        if (ok)
+            *ok = false;
         QV4::Scope scope(v4);
         QV4::ScopedArrayObject array(scope, object);
         if (!array)
@@ -364,163 +374,61 @@ public:
         return QMatrix4x4(matVals);
     }
 
-    template<typename T>
-    bool typedCreate(QQmlValueType *&v)
-    {
-        v = new T;
-        return true;
-    }
-
-    bool create(int type, QQmlValueType *&v)
+    const QMetaObject *getMetaObjectForMetaType(int type) Q_DECL_OVERRIDE
     {
         switch (type) {
         case QMetaType::QColor:
-            return typedCreate<QQuickColorValueType>(v);
+            return &QQuickColorValueType::staticMetaObject;
         case QMetaType::QFont:
-            return typedCreate<QQuickFontValueType>(v);
+            return &QQuickFontValueType::staticMetaObject;
         case QMetaType::QVector2D:
-            return typedCreate<QQuickVector2DValueType>(v);
+            return &QQuickVector2DValueType::staticMetaObject;
         case QMetaType::QVector3D:
-            return typedCreate<QQuickVector3DValueType>(v);
+            return &QQuickVector3DValueType::staticMetaObject;
         case QMetaType::QVector4D:
-            return typedCreate<QQuickVector4DValueType>(v);
+            return &QQuickVector4DValueType::staticMetaObject;
         case QMetaType::QQuaternion:
-            return typedCreate<QQuickQuaternionValueType>(v);
+            return &QQuickQuaternionValueType::staticMetaObject;
         case QMetaType::QMatrix4x4:
-            return typedCreate<QQuickMatrix4x4ValueType>(v);
+            return &QQuickMatrix4x4ValueType::staticMetaObject;
         default:
             break;
         }
 
-        return false;
+        return 0;
     }
 
-    template<typename T>
-    bool typedInit(void *data, size_t dataSize)
-    {
-        ASSERT_VALID_SIZE(dataSize, sizeof(T));
-        T *t = reinterpret_cast<T *>(data);
-        new (t) T();
-        return true;
-    }
-
-    bool init(int type, void *data, size_t dataSize)
+    bool init(int type, QVariant& dst) Q_DECL_OVERRIDE
     {
         switch (type) {
         case QMetaType::QColor:
-            return typedInit<QColor>(data, dataSize);
+            dst.setValue<QColor>(QColor());
+            return true;
         case QMetaType::QFont:
-            return typedInit<QFont>(data, dataSize);
+            dst.setValue<QFont>(QFont());
+            return true;
         case QMetaType::QVector2D:
-            return typedInit<QVector2D>(data, dataSize);
+            dst.setValue<QVector2D>(QVector2D());
+            return true;
         case QMetaType::QVector3D:
-            return typedInit<QVector3D>(data, dataSize);
+            dst.setValue<QVector3D>(QVector3D());
+            return true;
         case QMetaType::QVector4D:
-            return typedInit<QVector4D>(data, dataSize);
+            dst.setValue<QVector4D>(QVector4D());
+            return true;
         case QMetaType::QQuaternion:
-            return typedInit<QQuaternion>(data, dataSize);
+            dst.setValue<QQuaternion>(QQuaternion());
+            return true;
         case QMetaType::QMatrix4x4:
-            {
-            if (dataSize >= sizeof(QMatrix4x4))
-                return typedInit<QMatrix4x4>(data, dataSize);
-
-            // special case: init matrix-containing qvariant.
-            Q_ASSERT(dataSize >= sizeof(QVariant));
-            QVariant *matvar = reinterpret_cast<QVariant *>(data);
-            new (matvar) QVariant(QMatrix4x4());
+            dst.setValue<QMatrix4x4>(QMatrix4x4());
             return true;
-            }
         default: break;
         }
 
         return false;
     }
 
-    template<typename T>
-    bool typedDestroy(void *data, size_t dataSize)
-    {
-        ASSERT_VALID_SIZE(dataSize, sizeof(T));
-        T *t = reinterpret_cast<T *>(data);
-        t->~T();
-        return true;
-    }
-
-    bool destroy(int type, void *data, size_t dataSize)
-    {
-        switch (type) {
-        case QMetaType::QColor:
-            return typedDestroy<QColor>(data, dataSize);
-        case QMetaType::QFont:
-            return typedDestroy<QFont>(data, dataSize);
-        case QMetaType::QVector2D:
-            return typedDestroy<QVector2D>(data, dataSize);
-        case QMetaType::QVector3D:
-            return typedDestroy<QVector3D>(data, dataSize);
-        case QMetaType::QVector4D:
-            return typedDestroy<QVector4D>(data, dataSize);
-        case QMetaType::QQuaternion:
-            return typedDestroy<QQuaternion>(data, dataSize);
-        case QMetaType::QMatrix4x4:
-            {
-            if (dataSize >= sizeof(QMatrix4x4))
-                return typedDestroy<QMatrix4x4>(data, dataSize);
-
-            // special case: destroying matrix-containing qvariant.
-            Q_ASSERT(dataSize >= sizeof(QVariant));
-            QVariant *matvar = reinterpret_cast<QVariant *>(data);
-            matvar->~QVariant();
-            return true;
-            }
-        default: break;
-        }
-
-        return false;
-    }
-
-    template<typename T>
-    bool typedCopyConstruct(const void *src, void *dst, size_t dstSize)
-    {
-        ASSERT_VALID_SIZE(dstSize, sizeof(T));
-        const T *srcT = reinterpret_cast<const T *>(src);
-        T *destT = reinterpret_cast<T *>(dst);
-        new (destT) T(*srcT);
-        return true;
-    }
-
-    bool copy(int type, const void *src, void *dst, size_t dstSize)
-    {
-        switch (type) {
-        case QMetaType::QColor:
-            return typedCopyConstruct<QColor>(src, dst, dstSize);
-        case QMetaType::QFont:
-            return typedCopyConstruct<QFont>(src, dst, dstSize);
-        case QMetaType::QVector2D:
-            return typedCopyConstruct<QVector2D>(src, dst, dstSize);
-        case QMetaType::QVector3D:
-            return typedCopyConstruct<QVector3D>(src, dst, dstSize);
-        case QMetaType::QVector4D:
-            return typedCopyConstruct<QVector4D>(src, dst, dstSize);
-        case QMetaType::QQuaternion:
-            return typedCopyConstruct<QQuaternion>(src, dst, dstSize);
-        case QMetaType::QMatrix4x4:
-            {
-            if (dstSize >= sizeof(QMatrix4x4))
-                return typedCopyConstruct<QMatrix4x4>(src, dst, dstSize);
-
-            // special case: copying matrix into variant.
-            Q_ASSERT(dstSize >= sizeof(QVariant));
-            const QMatrix4x4 *srcMatrix = reinterpret_cast<const QMatrix4x4 *>(src);
-            QVariant *dstMatrixVar = reinterpret_cast<QVariant *>(dst);
-            new (dstMatrixVar) QVariant(*srcMatrix);
-            return true;
-            }
-        default: break;
-        }
-
-        return false;
-    }
-
-    bool create(int type, int argc, const void *argv[], QVariant *v)
+    bool create(int type, int argc, const void *argv[], QVariant *v) Q_DECL_OVERRIDE
     {
         switch (type) {
         case QMetaType::QFont: // must specify via js-object.
@@ -558,7 +466,11 @@ public:
             }
             break;
         case QMetaType::QMatrix4x4:
-            if (argc == 1) {
+            if (argc == 0) {
+                QMatrix4x4 m;
+                *v = QVariant(m);
+                return true;
+            } else if (argc == 1) {
                 const qreal *vals = reinterpret_cast<const qreal*>(argv[0]);
                 QMatrix4x4 m(vals[0], vals[1], vals[2], vals[3],
                              vals[4], vals[5], vals[6], vals[7],
@@ -583,7 +495,7 @@ public:
         return true;
     }
 
-    bool createFromString(int type, const QString &s, void *data, size_t dataSize)
+    bool createFromString(int type, const QString &s, void *data, size_t dataSize) Q_DECL_OVERRIDE
     {
         bool ok = false;
 
@@ -599,22 +511,14 @@ public:
         case QMetaType::QQuaternion:
             return createFromStringTyped<QQuaternion>(data, dataSize, quaternionFromString(s, &ok));
         case QMetaType::QMatrix4x4:
-            {
-            if (dataSize >= sizeof(QMatrix4x4))
-                return createFromStringTyped<QMatrix4x4>(data, dataSize, matrix4x4FromString(s, &ok));
-
-            Q_ASSERT(dataSize >= sizeof(QVariant));
-            QVariant *matVar = reinterpret_cast<QVariant *>(data);
-            new (matVar) QVariant(matrix4x4FromString(s, &ok));
-            return true;
-            }
+            return createFromStringTyped<QMatrix4x4>(data, dataSize, matrix4x4FromString(s, &ok));
         default: break;
         }
 
         return false;
     }
 
-    bool createStringFrom(int type, const void *data, QString *s)
+    bool createStringFrom(int type, const void *data, QString *s) Q_DECL_OVERRIDE
     {
         if (type == QMetaType::QColor) {
             const QColor *color = reinterpret_cast<const QColor *>(data);
@@ -625,7 +529,7 @@ public:
         return false;
     }
 
-    bool variantFromString(const QString &s, QVariant *v)
+    bool variantFromString(const QString &s, QVariant *v) Q_DECL_OVERRIDE
     {
         QColor c(s);
         if (c.isValid()) {
@@ -668,7 +572,7 @@ public:
         return false;
     }
 
-    bool variantFromString(int type, const QString &s, QVariant *v)
+    bool variantFromString(int type, const QString &s, QVariant *v) Q_DECL_OVERRIDE
     {
         bool ok = false;
 
@@ -711,9 +615,8 @@ public:
         return false;
     }
 
-    bool variantFromJsObject(int type, QQmlV4Handle object, QV8Engine *e, QVariant *v)
+    bool variantFromJsObject(int type, QQmlV4Handle object, QV4::ExecutionEngine *v4, QVariant *v) Q_DECL_OVERRIDE
     {
-        QV4::ExecutionEngine *v4 = QV8Engine::getV4(e);
         QV4::Scope scope(v4);
 #ifndef QT_NO_DEBUG
         QV4::ScopedObject obj(scope, object);
@@ -722,10 +625,10 @@ public:
         bool ok = false;
         switch (type) {
         case QMetaType::QFont:
-            *v = QVariant::fromValue(fontFromObject(object, e, &ok));
+            *v = QVariant::fromValue(fontFromObject(object, v4, &ok));
             break;
         case QMetaType::QMatrix4x4:
-            *v = QVariant::fromValue(matrix4x4FromObject(object, e, &ok));
+            *v = QVariant::fromValue(matrix4x4FromObject(object, v4, &ok));
         default: break;
         }
 
@@ -733,12 +636,12 @@ public:
     }
 
     template<typename T>
-    bool typedEqual(const void *lhs, const void *rhs)
+    bool typedEqual(const void *lhs, const QVariant& rhs)
     {
-        return (*(reinterpret_cast<const T *>(lhs)) == *(reinterpret_cast<const T *>(rhs)));
+        return (*(reinterpret_cast<const T *>(lhs)) == rhs.value<T>());
     }
 
-    bool equal(int type, const void *lhs, const void *rhs, size_t rhsSize)
+    bool equal(int type, const void *lhs, const QVariant &rhs) Q_DECL_OVERRIDE
     {
         switch (type) {
         case QMetaType::QColor:
@@ -754,14 +657,7 @@ public:
         case QMetaType::QQuaternion:
             return typedEqual<QQuaternion>(lhs, rhs);
         case QMetaType::QMatrix4x4:
-            {
-            if (rhsSize >= sizeof(QMatrix4x4))
-                return typedEqual<QMatrix4x4>(lhs, rhs);
-
-            Q_ASSERT(rhsSize >= sizeof(QVariant));
-            QMatrix4x4 rhsmat = reinterpret_cast<const QVariant *>(rhs)->value<QMatrix4x4>();
-            return typedEqual<QMatrix4x4>(lhs, &rhsmat);
-            }
+            return typedEqual<QMatrix4x4>(lhs, rhs);
         default: break;
         }
 
@@ -778,8 +674,9 @@ public:
         return true;
     }
 
-    bool store(int type, const void *src, void *dst, size_t dstSize)
+    bool store(int type, const void *src, void *dst, size_t dstSize) Q_DECL_OVERRIDE
     {
+        Q_UNUSED(dstSize);
         switch (type) {
         case QMetaType::QColor:
             {
@@ -789,81 +686,41 @@ public:
             new (color) QColor(QColor::fromRgba(*rgb));
             return true;
             }
-        case QMetaType::QFont:
-            return typedStore<QFont>(src, dst, dstSize);
-        case QMetaType::QVector2D:
-            return typedStore<QVector2D>(src, dst, dstSize);
-        case QMetaType::QVector3D:
-            return typedStore<QVector3D>(src, dst, dstSize);
-        case QMetaType::QVector4D:
-            return typedStore<QVector4D>(src, dst, dstSize);
-        case QMetaType::QQuaternion:
-            return typedStore<QQuaternion>(src, dst, dstSize);
-        case QMetaType::QMatrix4x4:
-            {
-            if (dstSize >= sizeof(QMatrix4x4))
-                return typedStore<QMatrix4x4>(src, dst, dstSize);
-
-            // special case: storing matrix into variant
-            // eg, QVMEMO QVMEVariant data cell is big enough to store
-            // QVariant, but not large enough to store QMatrix4x4.
-            Q_ASSERT(dstSize >= sizeof(QVariant));
-            const QMatrix4x4 *srcMat = reinterpret_cast<const QMatrix4x4 *>(src);
-            QVariant *dstMatVar = reinterpret_cast<QVariant *>(dst);
-            new (dstMatVar) QVariant(*srcMat);
-            return true;
-            }
-        default: break;
+            default: break;
         }
 
         return false;
     }
 
     template<typename T>
-    bool typedRead(int srcType, const void *src, size_t srcSize, int dstType, void *dst)
+    bool typedRead(const QVariant& src, int dstType, void *dst)
     {
         T *dstT = reinterpret_cast<T *>(dst);
-        if (srcType == dstType) {
-            ASSERT_VALID_SIZE(srcSize, sizeof(T));
-            const T *srcT = reinterpret_cast<const T *>(src);
-            *dstT = *srcT;
+        if (src.type() == static_cast<uint>(dstType)) {
+            *dstT = src.value<T>();
         } else {
             *dstT = T();
         }
         return true;
     }
 
-    bool read(int srcType, const void *src, size_t srcSize, int dstType, void *dst)
+    bool read(const QVariant &src, void *dst, int dstType) Q_DECL_OVERRIDE
     {
         switch (dstType) {
         case QMetaType::QColor:
-            return typedRead<QColor>(srcType, src, srcSize, dstType, dst);
+            return typedRead<QColor>(src, dstType, dst);
         case QMetaType::QFont:
-            return typedRead<QFont>(srcType, src, srcSize, dstType, dst);
+            return typedRead<QFont>(src, dstType, dst);
         case QMetaType::QVector2D:
-            return typedRead<QVector2D>(srcType, src, srcSize, dstType, dst);
+            return typedRead<QVector2D>(src, dstType, dst);
         case QMetaType::QVector3D:
-            return typedRead<QVector3D>(srcType, src, srcSize, dstType, dst);
+            return typedRead<QVector3D>(src, dstType, dst);
         case QMetaType::QVector4D:
-            return typedRead<QVector4D>(srcType, src, srcSize, dstType, dst);
+            return typedRead<QVector4D>(src, dstType, dst);
         case QMetaType::QQuaternion:
-            return typedRead<QQuaternion>(srcType, src, srcSize, dstType, dst);
+            return typedRead<QQuaternion>(src, dstType, dst);
         case QMetaType::QMatrix4x4:
-            {
-            if (srcSize >= sizeof(QMatrix4x4))
-                return typedRead<QMatrix4x4>(srcType, src, srcSize, dstType, dst);
-
-            // the source data may be stored in a QVariant.
-            QMatrix4x4 *dstMat = reinterpret_cast<QMatrix4x4 *>(dst);
-            if (srcType == dstType) {
-                Q_ASSERT(srcSize >= sizeof(QVariant));
-                const QVariant *srcMatVar = reinterpret_cast<const QVariant *>(src);
-                *dstMat = srcMatVar->value<QMatrix4x4>();
-            } else {
-                *dstMat = QMatrix4x4();
-            }
-            return true;
-            }
+            return typedRead<QMatrix4x4>(src, dstType, dst);
         default: break;
         }
 
@@ -871,51 +728,33 @@ public:
     }
 
     template<typename T>
-    bool typedWrite(const void *src, void *dst, size_t dstSize)
+    bool typedWrite(const void *src, QVariant& dst)
     {
-        ASSERT_VALID_SIZE(dstSize, sizeof(T));
         const T *srcT = reinterpret_cast<const T *>(src);
-        T *dstT = reinterpret_cast<T *>(dst);
-        if (*dstT != *srcT) {
-            *dstT = *srcT;
+        if (dst.value<T>() != *srcT) {
+            dst = *srcT;
             return true;
         }
         return false;
     }
 
-    bool write(int type, const void *src, void *dst, size_t dstSize)
+    bool write(int type, const void *src, QVariant& dst) Q_DECL_OVERRIDE
     {
         switch (type) {
         case QMetaType::QColor:
-            return typedWrite<QColor>(src, dst, dstSize);
+            return typedWrite<QColor>(src, dst);
         case QMetaType::QFont:
-            return typedWrite<QFont>(src, dst, dstSize);
+            return typedWrite<QFont>(src, dst);
         case QMetaType::QVector2D:
-            return typedWrite<QVector2D>(src, dst, dstSize);
+            return typedWrite<QVector2D>(src, dst);
         case QMetaType::QVector3D:
-            return typedWrite<QVector3D>(src, dst, dstSize);
+            return typedWrite<QVector3D>(src, dst);
         case QMetaType::QVector4D:
-            return typedWrite<QVector4D>(src, dst, dstSize);
+            return typedWrite<QVector4D>(src, dst);
         case QMetaType::QQuaternion:
-            return typedWrite<QQuaternion>(src, dst, dstSize);
+            return typedWrite<QQuaternion>(src, dst);
         case QMetaType::QMatrix4x4:
-            {
-            if (dstSize >= sizeof(QMatrix4x4))
-                return typedWrite<QMatrix4x4>(src, dst, dstSize);
-
-            // special case: storing matrix into variant
-            // eg, QVMEMO QVMEVariant data cell is big enough to store
-            // QVariant, but not large enough to store QMatrix4x4.
-            Q_ASSERT(dstSize >= sizeof(QVariant));
-            const QMatrix4x4 *srcMat = reinterpret_cast<const QMatrix4x4 *>(src);
-            QVariant *dstMatVar = reinterpret_cast<QVariant *>(dst);
-            QMatrix4x4 dstMatVal = dstMatVar->value<QMatrix4x4>();
-            if (dstMatVal != *srcMat) {
-                *dstMatVar = QVariant(*srcMat);
-                return true;
-            }
-            return false;
-            }
+            return typedWrite<QMatrix4x4>(src, dst);
         default: break;
         }
 
@@ -936,10 +775,19 @@ public:
 #ifndef QT_NO_IM
     QInputMethod *inputMethod()
     {
-        return qGuiApp->inputMethod();
+        QInputMethod *im = qGuiApp->inputMethod();
+        QQmlEngine::setObjectOwnership(im, QQmlEngine::CppOwnership);
+        return im;
     }
 #endif
 
+    QStyleHints *styleHints()
+    {
+        QStyleHints *sh = qGuiApp->styleHints();
+        QQmlEngine::setObjectOwnership(sh, QQmlEngine::CppOwnership);
+        return sh;
+    }
+
     QStringList fontFamilies()
     {
         QFontDatabase database;
