@@ -37,13 +37,12 @@
 QQmlDebugTestService::QQmlDebugTestService(const QString &s, float version, QObject *parent)
     : QQmlDebugService(s, version, parent)
 {
-    registerService();
 }
 
 void QQmlDebugTestService::messageReceived(const QByteArray &ba)
 {
     Q_ASSERT(QThread::currentThread() != thread());
-    QMetaObject::invokeMethod(this, "_sendMessage", Qt::QueuedConnection, Q_ARG(QByteArray, ba));
+    emit messageToClient(name(), ba);
 }
 
 void QQmlDebugTestService::stateAboutToBeChanged(QQmlDebugService::State)
@@ -56,8 +55,3 @@ void QQmlDebugTestService::stateChanged(State)
     Q_ASSERT(QThread::currentThread() != thread());
     emit stateHasChanged();
 }
-
-void QQmlDebugTestService::_sendMessage(const QByteArray &msg)
-{
-    QQmlDebugService::sendMessage(msg);
-}
