@@ -795,6 +795,8 @@ public:
     Q_INVOKABLE void method_QObject(QObject *a) { invoke(13); m_actuals << qVariantFromValue(a); }
     Q_INVOKABLE void method_QScriptValue(QJSValue a) { invoke(14); m_actuals << qVariantFromValue(a); }
     Q_INVOKABLE void method_intQScriptValue(int a, QJSValue b) { invoke(15); m_actuals << a << qVariantFromValue(b); }
+    Q_INVOKABLE QJSValue method_intQJSValue(int a, QJSValue b) { invoke(29); m_actuals << a << qVariantFromValue(b); return b.call(); }
+    Q_INVOKABLE QJSValue method_intQJSValue(int a, int b) { m_actuals << a << b; return QJSValue();} // Should never be called.
 
     Q_INVOKABLE void method_overload(int a) { invoke(16); m_actuals << a; }
     Q_INVOKABLE void method_overload(int a, int b) { invoke(17); m_actuals << a << b; }
@@ -1084,10 +1086,27 @@ protected:
     qreal m_p4;
 };
 
+
+class MyItemUsingRevisionedObject : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(MyRevisionedClass *revisioned READ revisioned)
+
+public:
+    MyItemUsingRevisionedObject() {
+        m_revisioned = new MyRevisionedClass;
+    }
+
+    MyRevisionedClass *revisioned() const { return m_revisioned; }
+private:
+    MyRevisionedClass *m_revisioned;
+};
+
 QML_DECLARE_TYPE(MyRevisionedBaseClassRegistered)
 QML_DECLARE_TYPE(MyRevisionedBaseClassUnregistered)
 QML_DECLARE_TYPE(MyRevisionedClass)
 QML_DECLARE_TYPE(MyRevisionedSubclass)
+QML_DECLARE_TYPE(MyItemUsingRevisionedObject)
 Q_DECLARE_METATYPE(MyQmlObject::MyType)
 
 
@@ -1253,7 +1272,7 @@ public:
     Q_INVOKABLE void addReference(QObject *other)
     {
         QQmlData *ddata = QQmlData::get(this);
-        assert(ddata);
+        Q_ASSERT(ddata);
         QV4::ExecutionEngine *v4 = ddata->jsWrapper.engine();
         Q_ASSERT(v4);
         QV4::Scope scope(v4);
