@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -65,11 +65,13 @@ private slots:
     void remote_data();
     void sourceSize();
     void sourceSizeChanges();
+    void sourceSizeChanges_intermediate();
     void sourceSizeReadOnly();
     void invalidSource();
     void qtbug_16520();
     void progressAndStatusChanges();
     void playingAndPausedChanges();
+    void noCaching();
 };
 
 void tst_qquickanimatedimage::cleanup()
@@ -209,16 +211,12 @@ void tst_qquickanimatedimage::mirror_notRunning()
 
     int frame = anim->currentFrame();
     bool playing = anim->isPlaying();
-    bool paused = anim->isPlaying();
+    bool paused = anim->isPaused();
 
     anim->setProperty("mirror", true);
     screenshot = window.grabWindow();
 
     screenshot.save("screen.png");
-#if defined(Q_OS_WIN)
-    // QTBUG-36717
-    QSKIP("This test is failing in the CI system under mysterious circumstances");
-#endif
     QCOMPARE(screenshot, expected);
 
     // mirroring should not change the current frame or playing status
@@ -251,12 +249,10 @@ void tst_qquickanimatedimage::remote()
     QFETCH(QString, fileName);
     QFETCH(bool, paused);
 
-    TestHTTPServer server;
-    QVERIFY2(server.listen(14449), qPrintable(server.errorString()));
-    server.serveDirectory(dataDirectory());
+    ThreadedTestHTTPServer server(dataDirectory());
 
     QQmlEngine engine;
-    QQmlComponent component(&engine, QUrl("http://127.0.0.1:14449/" + fileName));
+    QQmlComponent component(&engine, server.url(fileName));
     QTRY_VERIFY(component.isReady());
 
     QQuickAnimatedImage *anim = qobject_cast<QQuickAnimatedImage *>(component.create());
@@ -317,7 +313,7 @@ void tst_qquickanimatedimage::invalidSource()
 void tst_qquickanimatedimage::sourceSizeChanges()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(14449), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(dataDirectory());
 
     QQmlEngine engine;
@@ -334,56 +330,78 @@ void tst_qquickanimatedimage::sourceSizeChanges()
     // Local
     ctxt->setContextProperty("srcImage", QUrl(""));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Null);
-    QTRY_VERIFY(sourceSizeSpy.count() == 0);
+    QTRY_COMPARE(sourceSizeSpy.count(), 0);
 
     ctxt->setContextProperty("srcImage", testFileUrl("hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 1);
+    QTRY_COMPARE(sourceSizeSpy.count(), 1);
 
     ctxt->setContextProperty("srcImage", testFileUrl("hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 1);
+    QTRY_COMPARE(sourceSizeSpy.count(), 1);
 
     ctxt->setContextProperty("srcImage", testFileUrl("hearts_copy.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 1);
+    QTRY_COMPARE(sourceSizeSpy.count(), 1);
 
     ctxt->setContextProperty("srcImage", testFileUrl("colors.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 2);
+    QTRY_COMPARE(sourceSizeSpy.count(), 2);
 
     ctxt->setContextProperty("srcImage", QUrl(""));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Null);
-    QTRY_VERIFY(sourceSizeSpy.count() == 3);
+    QTRY_COMPARE(sourceSizeSpy.count(), 3);
 
     // Remote
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/hearts.gif"));
+    ctxt->setContextProperty("srcImage", server.url("/hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 4);
+    QTRY_COMPARE(sourceSizeSpy.count(), 4);
 
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/hearts.gif"));
+    ctxt->setContextProperty("srcImage", server.url("/hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 4);
+    QTRY_COMPARE(sourceSizeSpy.count(), 4);
 
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/hearts_copy.gif"));
+    ctxt->setContextProperty("srcImage", server.url("/hearts_copy.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 4);
+    QTRY_COMPARE(sourceSizeSpy.count(), 4);
 
-    ctxt->setContextProperty("srcImage", QUrl("http://127.0.0.1:14449/colors.gif"));
+    ctxt->setContextProperty("srcImage", server.url("/colors.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_VERIFY(sourceSizeSpy.count() == 5);
+    QTRY_COMPARE(sourceSizeSpy.count(), 5);
 
     ctxt->setContextProperty("srcImage", QUrl(""));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Null);
-    QTRY_VERIFY(sourceSizeSpy.count() == 6);
+    QTRY_COMPARE(sourceSizeSpy.count(), 6);
 
     delete anim;
 }
 
+void tst_qquickanimatedimage::sourceSizeChanges_intermediate()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQuick 2.0\nAnimatedImage { readonly property int testWidth: status === AnimatedImage.Ready ? sourceSize.width : -1; source: srcImage }", QUrl::fromLocalFile(""));
+    QTRY_VERIFY(component.isReady());
+    QQmlContext *ctxt = engine.rootContext();
+    ctxt->setContextProperty("srcImage", "");
+
+    QScopedPointer<QQuickAnimatedImage> anim(qobject_cast<QQuickAnimatedImage*>(component.create()));
+    QVERIFY(anim != 0);
+
+    ctxt->setContextProperty("srcImage", testFileUrl("hearts.gif"));
+    QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
+    QTRY_COMPARE(anim->property("testWidth").toInt(), anim->sourceSize().width());
+
+    ctxt->setContextProperty("srcImage", testFileUrl("hearts_copy.gif"));
+    QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
+    QTRY_COMPARE(anim->property("testWidth").toInt(), anim->sourceSize().width());
+}
+
+
 void tst_qquickanimatedimage::qtbug_16520()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(14449), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(dataDirectory());
 
     QQmlEngine engine;
@@ -395,9 +413,9 @@ void tst_qquickanimatedimage::qtbug_16520()
     QQuickAnimatedImage *anim = root->findChild<QQuickAnimatedImage*>("anim");
     QVERIFY(anim != 0);
 
-    anim->setProperty("source", "http://127.0.0.1:14449/stickman.gif");
-    QTRY_VERIFY(anim->opacity() == 0);
-    QTRY_VERIFY(anim->opacity() == 1);
+    anim->setProperty("source", server.urlString("/stickman.gif"));
+    QTRY_COMPARE(anim->opacity(), qreal(0));
+    QTRY_COMPARE(anim->opacity(), qreal(1));
 
     delete anim;
     delete root;
@@ -406,7 +424,7 @@ void tst_qquickanimatedimage::qtbug_16520()
 void tst_qquickanimatedimage::progressAndStatusChanges()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(14449), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(dataDirectory());
 
     QQmlEngine engine;
@@ -417,8 +435,8 @@ void tst_qquickanimatedimage::progressAndStatusChanges()
     component.setData(componentStr.toLatin1(), QUrl::fromLocalFile(""));
     QQuickImage *obj = qobject_cast<QQuickImage*>(component.create());
     QVERIFY(obj != 0);
-    QVERIFY(obj->status() == QQuickImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QCOMPARE(obj->status(), QQuickImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
 
     qRegisterMetaType<QQuickImageBase::Status>();
     QSignalSpy sourceSpy(obj, SIGNAL(sourceChanged(QUrl)));
@@ -427,33 +445,33 @@ void tst_qquickanimatedimage::progressAndStatusChanges()
 
     // Same image
     ctxt->setContextProperty("srcImage", testFileUrl("stickman.gif"));
-    QTRY_VERIFY(obj->status() == QQuickImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QTRY_COMPARE(obj->status(), QQuickImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
     QTRY_COMPARE(sourceSpy.count(), 0);
     QTRY_COMPARE(progressSpy.count(), 0);
     QTRY_COMPARE(statusSpy.count(), 0);
 
     // Loading local file
     ctxt->setContextProperty("srcImage", testFileUrl("colors.gif"));
-    QTRY_VERIFY(obj->status() == QQuickImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QTRY_COMPARE(obj->status(), QQuickImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
     QTRY_COMPARE(sourceSpy.count(), 1);
     QTRY_COMPARE(progressSpy.count(), 0);
     QTRY_COMPARE(statusSpy.count(), 1);
 
     // Loading remote file
-    ctxt->setContextProperty("srcImage", "http://127.0.0.1:14449/stickman.gif");
-    QTRY_VERIFY(obj->status() == QQuickImage::Loading);
-    QTRY_VERIFY(obj->progress() == 0.0);
-    QTRY_VERIFY(obj->status() == QQuickImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    ctxt->setContextProperty("srcImage", server.url("/stickman.gif"));
+    QTRY_COMPARE(obj->status(), QQuickImage::Loading);
+    QTRY_COMPARE(obj->progress(), 0.0);
+    QTRY_COMPARE(obj->status(), QQuickImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
     QTRY_COMPARE(sourceSpy.count(), 2);
     QTRY_VERIFY(progressSpy.count() > 1);
     QTRY_COMPARE(statusSpy.count(), 3);
 
     ctxt->setContextProperty("srcImage", "");
-    QTRY_VERIFY(obj->status() == QQuickImage::Null);
-    QTRY_VERIFY(obj->progress() == 0.0);
+    QTRY_COMPARE(obj->status(), QQuickImage::Null);
+    QTRY_COMPARE(obj->progress(), 0.0);
     QTRY_COMPARE(sourceSpy.count(), 3);
     QTRY_VERIFY(progressSpy.count() > 2);
     QTRY_COMPARE(statusSpy.count(), 4);
@@ -471,7 +489,7 @@ void tst_qquickanimatedimage::playingAndPausedChanges()
     component.setData(componentStr.toLatin1(), QUrl::fromLocalFile(""));
     QQuickAnimatedImage *obj = qobject_cast<QQuickAnimatedImage*>(component.create());
     QVERIFY(obj != 0);
-    QVERIFY(obj->status() == QQuickAnimatedImage::Null);
+    QCOMPARE(obj->status(), QQuickAnimatedImage::Null);
     QTRY_VERIFY(obj->isPlaying());
     QTRY_VERIFY(!obj->isPaused());
     QSignalSpy playingSpy(obj, SIGNAL(playingChanged()));
@@ -528,6 +546,39 @@ void tst_qquickanimatedimage::playingAndPausedChanges()
 
     delete obj;
 }
+
+void tst_qquickanimatedimage::noCaching()
+{
+    QQuickView window, window_nocache;
+    window.setSource(testFileUrl("colors.qml"));
+    window_nocache.setSource(testFileUrl("colors_nocache.qml"));
+    window.show();
+    window_nocache.show();
+    QTest::qWaitForWindowExposed(&window);
+    QTest::qWaitForWindowExposed(&window_nocache);
+
+    QQuickAnimatedImage *anim = qobject_cast<QQuickAnimatedImage *>(window.rootObject());
+    QVERIFY(anim);
+
+    QQuickAnimatedImage *anim_nocache = qobject_cast<QQuickAnimatedImage *>(window_nocache.rootObject());
+    QVERIFY(anim_nocache);
+
+    QCOMPARE(anim->frameCount(), anim_nocache->frameCount());
+
+    // colors.gif only has 3 frames so this should be fast
+    for (int loops = 0; loops <= 2; ++loops) {
+        for (int frame = 0; frame < anim->frameCount(); ++frame) {
+            anim->setCurrentFrame(frame);
+            anim_nocache->setCurrentFrame(frame);
+
+            QImage image_cache = window.grabWindow();
+            QImage image_nocache = window_nocache.grabWindow();
+
+            QCOMPARE(image_cache, image_nocache);
+        }
+    }
+}
+
 QTEST_MAIN(tst_qquickanimatedimage)
 
 #include "tst_qquickanimatedimage.moc"
